==> v3.3.1.tar.gz: bimba/app/src/main/java/xyz/apiote/bimba/czwek/search/ResultsActivity.kt <==
// SPDX-FileCopyrightText: Adam Evyčędo
//
// SPDX-License-Identifier: GPL-3.0-or-later

package xyz.apiote.bimba.czwek.search

import android.content.Context
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.View
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.view.WindowCompat
import androidx.recyclerview.widget.LinearLayoutManager
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.Runnable
import kotlinx.coroutines.launch
import xyz.apiote.bimba.czwek.R
import xyz.apiote.bimba.czwek.api.Error
import xyz.apiote.bimba.czwek.databinding.ActivityResultsBinding
import xyz.apiote.bimba.czwek.repo.OfflineRepository
import xyz.apiote.bimba.czwek.repo.OnlineRepository
import xyz.apiote.bimba.czwek.repo.Position
import xyz.apiote.bimba.czwek.repo.Queryable
import xyz.apiote.bimba.czwek.repo.TrafficResponseException
import xyz.apiote.bimba.czwek.settings.feeds.FeedsSettings

class ResultsActivity : AppCompatActivity(), LocationListener {
	enum class Mode {
		MODE_LOCATION, MODE_SEARCH, MODE_POSITION
	}

	private var _binding: ActivityResultsBinding? = null
	private val binding get() = _binding!!

	private lateinit var adapter: BimbaResultsAdapter

	private val handler = Handler(Looper.getMainLooper())
	private var runnable = Runnable {}

	override fun onCreate(savedInstanceState: Bundle?) {
		super.onCreate(savedInstanceState)
		_binding = ActivityResultsBinding.inflate(layoutInflater)
		setContentView(binding.root)

		binding.resultsRecycler.layoutManager = LinearLayoutManager(this)
		adapter = BimbaResultsAdapter(layoutInflater, this, listOf())
		binding.resultsRecycler.adapter = adapter
		setSupportActionBar(binding.topAppBar)

		WindowCompat.setDecorFitsSystemWindows(window, false)



		when (getMode()) {
			Mode.MODE_LOCATION -> {
				supportActionBar?.title = getString(R.string.stops_nearby)
				locate()
			}

			Mode.MODE_POSITION -> {
				val query = intent.extras?.getString("query")
				val lat = intent.extras?.getDouble("lat")
				val lon = intent.extras?.getDouble("lon")
				supportActionBar?.title = getString(R.string.stops_near_code, query)
				getQueryablesByLocation(Position(lat!!, lon!!), this)
			}

			Mode.MODE_SEARCH -> {
				val query = intent.extras?.getString("query")!!
				supportActionBar?.title = getString(R.string.results_for, query)
				getQueryablesByQuery(query, this)
			}
		}
	}

	private fun getMode(): Mode {
		return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
			intent.extras!!.getSerializable("mode", Mode::class.java)!!
		} else {
			@Suppress("DEPRECATION")
			intent.extras!!.get("mode") as Mode
		}
	}

	private fun locate() {
		try {
			val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
			locationManager.requestLocationUpdates(
				LocationManager.GPS_PROVIDER, 1000 * 60 * 10, 100f, this
			)
			handler.removeCallbacks(runnable)
			runnable = Runnable {
				showError(Error(0, R.string.error_gps, R.drawable.error_gps))
			}
			handler.postDelayed(runnable, 60 * 1000)
			locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER)
				?.let { onLocationChanged(it) }
		} catch (_: SecurityException) {
			// this won’t happen because we don’t start this activity without location permission
		}
	}

	override fun onLocationChanged(location: Location) {
		handler.removeCallbacks(runnable)
		getQueryablesByLocation(Position(location.latitude, location.longitude), this)
	}

	override fun onResume() {
		super.onResume()
		if (getMode() == Mode.MODE_LOCATION) {
			locate()
		}
	}

	override fun onPause() {
		super.onPause()
		val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
		locationManager.removeUpdates(this)
		handler.removeCallbacks(runnable)
	}

	override fun onDestroy() {
		super.onDestroy()
		val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
		locationManager.removeUpdates(this)
		handler.removeCallbacks(runnable)
	}

	private suspend fun getFeeds() {
		if (adapter.feeds.isNullOrEmpty()) {
			adapter.feeds = OfflineRepository().getFeeds(this)
		}
		if (adapter.feedsSettings == null) {
			adapter.feedsSettings = FeedsSettings.load(this)
		}
	}

	private fun getQueryablesByQuery(query: String, context: Context) {
		MainScope().launch {
			try {
				val repository = OnlineRepository()
				val result = repository.queryQueryables(query, context)
				getFeeds()
				updateItems(result)
			} catch (e: TrafficResponseException) {
				Log.w("Suggestion", "$e")
				showError(e.error)
			}
		}
	}

	private fun getQueryablesByLocation(position: Position, context: Context) {
		MainScope().launch {
			try {
				val repository = OnlineRepository()
				val result = repository.locateQueryables(position, context)
				getFeeds()
				updateItems(result)
			} catch (e: TrafficResponseException) {
				Log.w("Suggestion", "$e")
				showError(e.error)
			}
		}
	}

	private fun showError(error: Error) {
		binding.resultsProgress.visibility = View.GONE
		binding.resultsRecycler.visibility = View.GONE
		binding.errorImage.visibility = View.VISIBLE
		binding.errorText.visibility = View.VISIBLE

		binding.errorText.text = getString(error.stringResource)
		binding.errorImage.setImageDrawable(AppCompatResources.getDrawable(this, error.imageResource))
	}

	private fun updateItems(queryables: List<Queryable>?) {
		binding.resultsProgress.visibility = View.GONE
		adapter.update(queryables)
		if (queryables.isNullOrEmpty()) {
			binding.errorImage.visibility = View.VISIBLE
			binding.errorText.visibility = View.VISIBLE
			binding.resultsRecycler.visibility = View.GONE

			binding.errorText.text = getString(R.string.error_404)
			binding.errorImage.setImageDrawable(
				AppCompatResources.getDrawable(
					this,
					R.drawable.error_search
				)
			)
		} else {
			if (queryables.size == 1 && getMode() == Mode.MODE_SEARCH) {
				adapter.click(0)
			}
			binding.resultsOverlay.visibility = View.GONE
			binding.errorImage.visibility = View.GONE
			binding.errorText.visibility = View.GONE
			binding.resultsRecycler.visibility = View.VISIBLE
		}
	}
}

==> v3.4.0.tar.gz: bimba/app/src/main/java/xyz/apiote/bimba/czwek/search/ResultsActivity.kt <==
// SPDX-FileCopyrightText: Adam Evyčędo
//
// SPDX-License-Identifier: GPL-3.0-or-later

package xyz.apiote.bimba.czwek.search

import android.content.Context
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.view.View
import android.view.ViewGroup.MarginLayoutParams
import androidx.activity.enableEdgeToEdge
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.content.res.AppCompatResources
import androidx.core.view.ViewCompat
import androidx.core.view.WindowInsetsCompat
import androidx.core.view.updateLayoutParams
import androidx.core.view.updatePadding
import androidx.recyclerview.widget.LinearLayoutManager
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.Runnable
import kotlinx.coroutines.launch
import xyz.apiote.bimba.czwek.R
import xyz.apiote.bimba.czwek.api.Error
import xyz.apiote.bimba.czwek.databinding.ActivityResultsBinding
import xyz.apiote.bimba.czwek.repo.OfflineRepository
import xyz.apiote.bimba.czwek.repo.OnlineRepository
import xyz.apiote.bimba.czwek.repo.Position
import xyz.apiote.bimba.czwek.repo.Queryable
import xyz.apiote.bimba.czwek.repo.TrafficResponseException
import xyz.apiote.bimba.czwek.settings.feeds.FeedsSettings

class ResultsActivity : AppCompatActivity(), LocationListener {
	enum class Mode {
		MODE_LOCATION, MODE_SEARCH, MODE_POSITION
	}

	private var _binding: ActivityResultsBinding? = null
	private val binding get() = _binding!!

	private lateinit var adapter: BimbaResultsAdapter

	private val handler = Handler(Looper.getMainLooper())
	private var runnable = Runnable {}

	override fun onCreate(savedInstanceState: Bundle?) {
		enableEdgeToEdge()
		super.onCreate(savedInstanceState)
		_binding = ActivityResultsBinding.inflate(layoutInflater)
		setContentView(binding.root)

		ViewCompat.setOnApplyWindowInsetsListener(binding.resultsRecycler) { v, windowInsets ->

			windowInsets.displayCutout?.safeInsetLeft?.let {
				v.updateLayoutParams<MarginLayoutParams> {
					leftMargin = it
				}
			}
			val insets = windowInsets.getInsets(WindowInsetsCompat.Type.systemBars())
			v.updatePadding(right = insets.right, left = insets.left)
			windowInsets
		}
		ViewCompat.setOnApplyWindowInsetsListener(binding.topAppBar) { v, windowInsets ->
			val insets = windowInsets.getInsets(WindowInsetsCompat.Type.systemBars())
			v.updatePadding(right = insets.right, left = insets.left)
			windowInsets
		}

		binding.resultsRecycler.layoutManager = LinearLayoutManager(this)
		adapter = BimbaResultsAdapter(layoutInflater, this, listOf())
		binding.resultsRecycler.adapter = adapter

		when (getMode()) {
			Mode.MODE_LOCATION -> {
				supportActionBar?.title = getString(R.string.stops_nearby)
				locate()
			}

			Mode.MODE_POSITION -> {
				val query = intent.extras?.getString("query")
				val lat = intent.extras?.getDouble("lat")
				val lon = intent.extras?.getDouble("lon")
				supportActionBar?.title = getString(R.string.stops_near_code, query)
				getQueryablesByLocation(Position(lat!!, lon!!), this)
			}

			Mode.MODE_SEARCH -> {
				val query = intent.extras?.getString("query")!!
				supportActionBar?.title = getString(R.string.results_for, query)
				getQueryablesByQuery(query, this)
			}
		}
	}

	private fun getMode(): Mode {
		return if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
			intent.extras!!.getSerializable("mode", Mode::class.java)!!
		} else {
			@Suppress("DEPRECATION")
			intent.extras!!.get("mode") as Mode
		}
	}

	private fun locate() {
		try {
			val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
			locationManager.requestLocationUpdates(
				LocationManager.GPS_PROVIDER, 1000 * 60 * 10, 100f, this
			)
			handler.removeCallbacks(runnable)
			runnable = Runnable {
				showError(Error(0, R.string.error_gps, R.drawable.error_gps))
			}
			handler.postDelayed(runnable, 60 * 1000)
			locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER)
				?.let { onLocationChanged(it) }
		} catch (_: SecurityException) {
			// this won’t happen because we don’t start this activity without location permission
		}
	}

	override fun onLocationChanged(location: Location) {
		handler.removeCallbacks(runnable)
		getQueryablesByLocation(Position(location.latitude, location.longitude), this)
	}

	override fun onResume() {
		super.onResume()
		if (getMode() == Mode.MODE_LOCATION) {
			locate()
		}
	}

	override fun onPause() {
		super.onPause()
		val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
		locationManager.removeUpdates(this)
		handler.removeCallbacks(runnable)
	}

	override fun onDestroy() {
		super.onDestroy()
		val locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
		locationManager.removeUpdates(this)
		handler.removeCallbacks(runnable)
	}

	private suspend fun getFeeds() {
		if (adapter.feeds.isNullOrEmpty()) {
			val repository = OfflineRepository(this)
			adapter.feeds = repository.getFeeds(this)
			repository.close()
		}
		if (adapter.feedsSettings == null) {
			adapter.feedsSettings = FeedsSettings.load(this)
		}
	}

	private fun getQueryablesByQuery(query: String, context: Context) {
		MainScope().launch {
			try {
				val repository = OnlineRepository()
				val result = repository.queryQueryables(query, context)
				getFeeds()
				updateItems(result)
			} catch (e: TrafficResponseException) {
				Log.w("Suggestion", "$e")
				showError(e.error)
			}
		}
	}

	private fun getQueryablesByLocation(position: Position, context: Context) {
		MainScope().launch {
			try {
				val repository = OnlineRepository()
				val result = repository.locateQueryables(position, context)
				getFeeds()
				updateItems(result)
			} catch (e: TrafficResponseException) {
				Log.w("Suggestion", "$e")
				showError(e.error)
			}
		}
	}

	private fun showError(error: Error) {
		binding.resultsProgress.visibility = View.GONE
		binding.resultsRecycler.visibility = View.GONE
		binding.errorImage.visibility = View.VISIBLE
		binding.errorText.visibility = View.VISIBLE

		binding.errorText.text = getString(error.stringResource)
		binding.errorImage.setImageDrawable(AppCompatResources.getDrawable(this, error.imageResource))
	}

	private fun updateItems(queryables: List<Queryable>?) {
		binding.resultsProgress.visibility = View.GONE
		adapter.update(queryables)
		if (queryables.isNullOrEmpty()) {
			binding.errorImage.visibility = View.VISIBLE
			binding.errorText.visibility = View.VISIBLE
			binding.resultsRecycler.visibility = View.GONE

			binding.errorText.text = getString(R.string.error_404)
			binding.errorImage.setImageDrawable(
				AppCompatResources.getDrawable(
					this,
					R.drawable.error_search
				)
			)
		} else {
			if (queryables.size == 1 && getMode() == Mode.MODE_SEARCH) {
				adapter.click(0)
			}
			binding.resultsOverlay.visibility = View.GONE
			binding.errorImage.visibility = View.GONE
			binding.errorText.visibility = View.GONE
			binding.resultsRecycler.visibility = View.VISIBLE
		}
	}
}
